// C/C++ File
// AUTHOR:   foton
// FILE:     main.c
// ROLE:     to rule them all
// CREATED:  2017-01-30 21:51:28
// MODIFIED: 2017-02-13 20:30:48

#include "fdf.h"

float	length(t_vec v)
{
	return (sqrt(v.x * v.x + v.y * v.y + v.z * v.z));
}

void	normalize(t_vec *vec)
{
	float len;

	len = length(*vec);
	if (len > 0)
	{
		len = 1 / len;
		vec->x *= len;
		vec->y *= len;
		vec->z *= len;
	}
}

float	dot(t_vec v1, t_vec v2)
{
	return (v1.x * v2.x + v1.y * v2.y + v1.z * v2.z);
}

t_vec	*cross(t_vec *v1, t_vec *v2)
{
	t_vec *v3;

	v3 = NULL;
	if ((v3 = (t_vec *)malloc(sizeof(t_vec))))
	{
		v3->x = v1->y * v2->z - v1->z * v2->y;
		v3->y = v1->z * v2->x - v1->x * v2->z;
		v3->z = v1->x * v2->y - v1->y * v2->z;
	}
	return (v3);
}

t_vec	*vec_plus(t_vec *v1, t_vec *v2)
{
	t_vec *v3;

	v3 = NULL;
	if ((v3 = (t_vec *)malloc(sizeof(t_vec))))
	{
		v3->x = v1->x + v2->x;
		v3->y = v1->y + v2->y;
		v3->z = v1->z + v2->z;
	}
	return (v3);
}

t_vec	*vec_minux(t_vec *v1, t_vec *v2)
{
	t_vec *v3;

	v3 = NULL;
	if ((v3 = (t_vec *)malloc(sizeof(t_vec))))
	{
		v3->x = v1->x - v2->x;
		v3->y = v1->y - v2->y;
		v3->z = v1->z - v2->z;
	}
	return (v3);
}

t_vec	*scalar_mult(t_vec *v1, float i)
{
	t_vec *v3;

	v3 = NULL;
	if ((v3 = (t_vec *)malloc(sizeof(t_vec))))
	{
		v3->x = v1->x * i;
		v3->y = v1->y * i;
		v3->z = v1->z * i;
	}
	return (v3);
}

t_mat4	*mat4_init(void)
{
	t_mat4	*mat;

	mat = NULL;
	if ((mat = (t_mat4 *)malloc(sizeof(t_mat4))))
	{
		mat->mx[0][0] = 1.f;
		mat->mx[1][1] = 1.f;
		mat->mx[2][2] = 1.f;
		mat->mx[3][3] = 1.f;
	}
	return (mat);
}

t_mat4	*mat4_mult(t_mat4 *m1, t_mat4 *m2)
{
	t_mat4	*m3;
	int 	i;
	int 	j;

	i = 0;
	m3 = NULL;
	if ((m3 = (t_mat4 *)malloc(sizeof(t_mat4))))
	{
		while(i < 4)
		{
			j = 0;
			while (j < 4)
			{
				m3->mx[i][j] = m1->mx[i][0] * m2->mx[0][j] +
								 m1->mx[i][1] * m2->mx[1][j] +
								 m1->mx[i][2] * m2->mx[2][j] +
								 m1->mx[i][3] * m2->mx[3][j];
				j++;
			}
			i++;
		}
	}
	return (m3);
}

void	mat4_z_rot(t_mat4 *m, float angle)
{
	angle = angle * M_1_PI / 180;
	m->mx[0][0] = cos(angle);
	m->mx[0][1] = sin(angle);
	m->mx[0][2] = 0;
	m->mx[1][0] = -(sin(angle));
	m->mx[1][1] = cos(angle);
	m->mx[1][2] = 0;
	m->mx[2][0] = 0;
	m->mx[2][1] = 0;
	m->mx[2][2] = 1;
}

void	mat4_x_rot(t_mat4 *m, float angle)
{
	angle = angle * M_1_PI / 180;
	m->mx[0][0] = 1;
	m->mx[0][1] = 0;
	m->mx[0][2] = 0;
	m->mx[1][0] = 0;
	m->mx[1][1] = cos(angle);
	m->mx[1][2] = sin(angle);
	m->mx[2][0] = 0;
	m->mx[2][1] = -(sin(angle));
	m->mx[2][2] = cos(angle);
}

void	mat4_y_rot(t_mat4 *m, float angle)
{
	angle = angle * M_1_PI / 180;
	m->mx[0][0] = cos(angle);
	m->mx[0][1] = 0;
	m->mx[0][2] = -(sin(angle));
	m->mx[1][0] = 0;
	m->mx[1][1] = 1;
	m->mx[1][2] = 0;
	m->mx[2][0] = sin(angle);
	m->mx[2][1] = 0;
	m->mx[2][2] = cos(angle);
}

void	vec_mat_mult(t_mat4 *m, t_vec *v, t_vec *v2)
{
	float vec4;

	v2->x = v->x * m->mx[0][0] + v->y * m->mx[1][0] + v->z * m->mx[2][0] +
			m->mx[3][0];
	v2->y = v->x * m->mx[0][1] + v->y * m->mx[1][1] + v->z * m->mx[2][1] +
			m->mx[3][1];
	v2->z = v->x * m->mx[0][2] + v->y * m->mx[1][2] + v->z * m->mx[2][2] +
			m->mx[3][2];
	vec4 = v->x * m->mx[0][3] + v->y * m->mx[1][3] + v->z * m->mx[2][3] +
			m->mx[3][3];
	if (vec4 != 1 && vec4 != 0)
	{
		v2->x = v2->x / vec4;
		v2->y = v2->y / vec4;
		v2->z = v2->z / vec4;
	}
}

void	test_line(t_mlx mlx)
{
	t_vec p1;
	t_vec p2;
	int		temp;

	p1.color = 0xC100E5;
	p2.color = 0xC100E5;
	for (p2.y = 0; p2.y <= 500; p2.y += 4)
	{
		p1.x = 0;
		p1.y = 0;
		p2.x = 500;
		line(&p1, &p2, &mlx);
	}
	for (p2.x = 500; p2.x >= 0; p2.x -= 4)
	{
		p1.x = 0;
		p1.y = 0;
		p2.y = 500;
		temp = p2.x;
		line(&p1, &p2, &mlx);
		p2.x = temp;
	}
	p1.color = 0x1EBF00;
	p2.color = 0x1EBF00;
	for (p2.y = 0; p2.y <= 500; p2.y += 4)
	{
		p1.x = 500;
		p1.y = 0;
		p2.x = 0;
		temp = p2.y;
		line(&p1, &p2, &mlx);
		p2.y = temp;
	}
	for (p2.x = 0; p2.x <= 500; p2.x += 4)
	{
		p1.x = 500;
		p1.y = 0;
		p2.y = 500;
		temp = p2.x;
		line(&p1, &p2, &mlx);
		p2.x = temp;
	}
}

void	circle(t_mlx m)
{
	t_vec	p1;
	t_vec	p2;
	double angle;


	p1.color = 0xFFFFFF;
	p2.color = 0xFFFFFF;
	for (int i = 0; i < 360; i += 4)
	{
		p1.x = 250;
		p1.y = 250;
		angle = i * M_PI / 180;
		p2.x = p1.x + 200 * sin(angle);
		p2.y = p1.y + 200 * cos(angle);
		line(&p1, &p2, &m);
	}
}

int		main(int argc, char **argv)
{
	t_mlx	m;
	int 	fd;
	char 	*line;

	if (argc != 2)
		return (0);
	fd = open(argv[1], O_RDONLY);
	// add new get_next_line
	while (get_next_line(fd, &line))
	{
		ft_putstr(line);
		ft_putstr("\n");
	}


	m.mlx = mlx_init();
	m.win = mlx_new_window(m.mlx, 500, 500, "fdf");
	test_line(m);
	circle(m);
	mlx_loop(m.mlx);
	return (0);
}
